@@ -73,15 +73,15 @@ class CProblem extends CApiService {
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectAcknowledges'		=> null,
 			'selectTags'				=> null,
-			'countOutput'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
@@ -289,7 +289,7 @@ class CProblem extends CApiService {
 		// recent
 		if ($options['recent'] !== null && $options['recent']) {
 			$config = select_config();
-			$ok_events_from = time() - $config['ok_period'];
+			$ok_events_from = time() - timeUnitToSeconds($config['ok_period']);
 
 			$sqlParts['where'][] = '(p.r_eventid IS NULL OR p.r_clock>'.$ok_events_from.')';
 		}
@@ -336,7 +336,7 @@ class CProblem extends CApiService {
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($event = DBfetch($res)) {
-			if ($options['countOutput'] !== null) {
+			if ($options['countOutput']) {
 				$result = $event['rowscount'];
 			}
 			else {
@@ -344,7 +344,7 @@ class CProblem extends CApiService {
 			}
 		}
 
-		if ($options['countOutput'] !== null) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
@@ -354,7 +354,7 @@ class CProblem extends CApiService {
 		}
 
 		// removing keys (hash -> array)
-		if ($options['preservekeys'] === null) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
@@ -431,15 +431,24 @@ class CProblem extends CApiService {
 
 		// Adding event tags.
 		if ($options['selectTags'] !== null && $options['selectTags'] != API_OUTPUT_COUNT) {
-			$tags = API::getApiService()->select('problem_tag', [
+			$tags_options = [
 				'output' => $this->outputExtend($options['selectTags'], ['eventid']),
-				'filter' => ['eventid' => $eventids],
-				'preservekeys' => true
-			]);
+				'filter' => ['eventid' => $eventids]
+			];
+			$tags = DBselect(DB::makeSql('problem_tag', $tags_options));
 
-			$relationMap = $this->createRelationMap($tags, 'eventid', 'problemtagid');
-			$tags = $this->unsetExtraFields($tags, ['problemtagid', 'eventid'], []);
-			$result = $relationMap->mapMany($result, $tags, 'tags');
+			foreach ($result as &$event) {
+				$event['tags'] = [];
+			}
+			unset($event);
+
+			while ($tag = DBfetch($tags)) {
+				$event = &$result[$tag['eventid']];
+
+				unset($tag['problemtagid'], $tag['eventid']);
+				$event['tags'][] = $tag;
+			}
+			unset($event);
 		}
 
 		return $result;
