@@ -181,16 +181,10 @@ function get_icon($type, $params = []) {
 
 			return $icon;
 
-		case 'dashconf':
-			$icon = (new CRedirectButton(SPACE, 'dashconf.php'))
-				->addClass(ZBX_STYLE_BTN_CONF)
-				->setTitle(_('Configure'));
-
-			if ($params['enabled']) {
-				$icon = [$icon, (new CDiv())->addClass(ZBX_STYLE_ACTIVE_INDIC)];
-			}
-
-			return $icon;
+		case 'action':
+			return (new CRedirectButton(SPACE, null))
+				->addClass(ZBX_STYLE_BTN_ACTION)
+				->setTitle(_('Actions'));
 
 		case 'screenconf':
 			return (new CRedirectButton(SPACE, null))
