@@ -162,20 +162,19 @@ $themes = array_merge([THEME_DEFAULT => _('System default')], Z::getThemes());
 $userFormList->addRow(_('Theme'), new CComboBox('theme', $this->data['theme'], null, $themes));
 
 // append auto-login & auto-logout to form list
-$autologoutCheckBox = (new CCheckBox('autologout_visible'))->setChecked(isset($this->data['autologout']));
-if (isset($this->data['autologout'])) {
-	$autologoutTextBox = (new CNumericBox('autologout', $this->data['autologout'], 4))
-		->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH);
+$autologoutCheckBox = (new CCheckBox('autologout_visible'))->setChecked($data['autologout_visible']);
+if ($data['autologout_visible']) {
+	$autologoutTextBox = (new CTextBox('autologout', $data['autologout']))->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
 }
 else {
-	$autologoutTextBox = (new CNumericBox('autologout', 900, 4))
-		->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+	$autologoutTextBox = (new CTextBox('autologout', DB::getDefault('users', 'autologout')))
+		->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 		->setAttribute('disabled', 'disabled');
 }
 
 if ($this->data['alias'] != ZBX_GUEST_USER) {
 	$userFormList->addRow(_('Auto-login'), (new CCheckBox('autologin'))->setChecked($this->data['autologin']));
-	$userFormList->addRow(_('Auto-logout (min 90 seconds)'), [
+	$userFormList->addRow(_('Auto-logout'), [
 		$autologoutCheckBox,
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 		$autologoutTextBox
@@ -183,8 +182,8 @@ if ($this->data['alias'] != ZBX_GUEST_USER) {
 }
 
 $userFormList
-	->addRow(_('Refresh (in seconds)'),
-		(new CNumericBox('refresh', $this->data['refresh'], 4))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+	->addRow(_('Refresh'),
+		(new CTextBox('refresh', $data['refresh']))->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 	)
 	->addRow(_('Rows per page'),
 		(new CNumericBox('rows_per_page', $this->data['rows_per_page'], 6))
@@ -244,7 +243,9 @@ if (uint_in_array(CWebUser::$data['type'], [USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SU
 			(new CRow([
 				$media['description'],
 				$media['sendto'],
-				$media['period'],
+				(new CDiv($media['period']))
+					->setAttribute('style', 'max-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
+					->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS),
 				$mediaSeverity,
 				$status,
 				(new CCol(
@@ -283,9 +284,9 @@ if ($this->data['is_profile']) {
 	$userMessagingFormList->addRow(_('Frontend messaging'),
 		(new CCheckBox('messages[enabled]'))->setChecked($this->data['messages']['enabled'] == 1)
 	);
-	$userMessagingFormList->addRow(_('Message timeout (seconds)'),
-		(new CNumericBox('messages[timeout]', $this->data['messages']['timeout'], 5))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH),
+	$userMessagingFormList->addRow(_('Message timeout'),
+		(new CTextBox('messages[timeout]', $this->data['messages']['timeout']))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH),
 		'timeout_row'
 	);
 
@@ -306,11 +307,9 @@ if ($this->data['is_profile']) {
 
 	$triggersTable = (new CTable())
 		->addRow([
-			new CLabel([
-				(new CCheckBox('messages[triggers.recovery]'))
-					->setChecked($this->data['messages']['triggers.recovery'] == 1),
-				_('Recovery')], 'messages[triggers.recovery]'
-			),
+			(new CCheckBox('messages[triggers.recovery]'))
+				->setLabel(_('Recovery'))
+				->setChecked($this->data['messages']['triggers.recovery'] == 1),
 			[
 				$soundList,
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -342,11 +341,9 @@ if ($this->data['is_profile']) {
 		}
 
 		$triggersTable->addRow([
-			new CLabel([
-				(new CCheckBox('messages[triggers.severities]['.$severity.']'))
-					->setChecked(isset($this->data['messages']['triggers.severities'][$severity])),
-				getSeverityName($severity, $this->data['config'])], 'messages[triggers.severities]['.$severity.']'
-			),
+			(new CCheckBox('messages[triggers.severities]['.$severity.']'))
+				->setLabel(getSeverityName($severity, $this->data['config']))
+				->setChecked(isset($this->data['messages']['triggers.severities'][$severity])),
 			[
 				$soundList,
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
