@@ -20,6 +20,7 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 
 $page['title'] = _('Availability report');
@@ -52,9 +53,13 @@ CProfile::update('web.avail_report.mode', $availabilityReportMode, PROFILE_TYPE_
  * Permissions
  */
 if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
-	if (getRequest('hostgroupid') && !API::HostGroup()->isReadable([$_REQUEST['hostgroupid']])
-			|| getRequest('filter_groupid') && !API::HostGroup()->isReadable([$_REQUEST['filter_groupid']])
-			|| getRequest('filter_hostid') && !API::Host()->isReadable([$_REQUEST['filter_hostid']])) {
+	if (getRequest('hostgroupid') && !isReadableHostGroups([getRequest('hostgroupid')])) {
+		access_deny();
+	}
+	if (getRequest('filter_groupid') && !isReadableHostGroups([getRequest('filter_groupid')])) {
+		access_deny();
+	}
+	if (getRequest('filter_hostid') && !isReadableTemplates([getRequest('filter_hostid')])) {
 		access_deny();
 	}
 	if (getRequest('tpl_triggerid')) {
@@ -69,12 +74,14 @@ if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
 	}
 }
 else {
-	if (getRequest('filter_groupid') && !API::HostGroup()->isReadable([$_REQUEST['filter_groupid']])
-			|| getRequest('filter_hostid') && !API::Host()->isReadable([$_REQUEST['filter_hostid']])) {
+	if (getRequest('filter_groupid') && !isReadableHostGroups([getRequest('filter_groupid')])) {
+		access_deny();
+	}
+	if (getRequest('filter_hostid') && !isReadableHosts([getRequest('filter_hostid')])) {
 		access_deny();
 	}
 }
-if (getRequest('triggerid') && !API::Trigger()->isReadable([$_REQUEST['triggerid']])) {
+if (getRequest('triggerid') && !isReadableTriggers([getRequest('triggerid')])) {
 	access_deny();
 }
 
@@ -226,34 +233,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			'with_triggers' => true,
 			'preservekeys' => true
 		]);
-
-		$parents = [];
-		$parent_name = '';
-		foreach ($groups as $group) {
-			$parent = explode('/', $group['name']);
-			if (count($parent) > 1) {
-				array_pop($parent);
-				foreach ($parent as $sub_parent) {
-					if ($parent_name === '') {
-						$parent_name = $sub_parent;
-					}
-					else {
-						$parent_name .= '/'.$sub_parent;
-					}
-					$parents[] = $parent_name;
-				}
-			}
-		}
-
-		if ($parents) {
-			$parent_groups = API::HostGroup()->get([
-				'output' => ['groupid', 'name'],
-				'filter' => ['name' => $parents],
-				'preservekeys' => true
-			]);
-
-			$groups = array_replace($groups, $parent_groups);
-		}
+		$groups = CPageFilter::enrichParentGroups($groups);
 
 		order_result($groups, 'name');
 
@@ -337,34 +317,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			'monitored_hosts' => true,
 			'preservekeys' => true
 		]);
-
-		$parents = [];
-		$parent_name = '';
-		foreach ($groups as $group) {
-			$parent = explode('/', $group['name']);
-			if (count($parent) > 1) {
-				array_pop($parent);
-				foreach ($parent as $sub_parent) {
-					if ($parent_name === '') {
-						$parent_name = $sub_parent;
-					}
-					else {
-						$parent_name .= '/'.$sub_parent;
-					}
-					$parents[] = $parent_name;
-				}
-			}
-		}
-
-		if ($parents) {
-			$parent_groups = API::HostGroup()->get([
-				'output' => ['groupid', 'name'],
-				'filter' => ['name' => $parents],
-				'preservekeys' => true
-			]);
-
-			$groups = array_replace($groups, $parent_groups);
-		}
+		$groups = CPageFilter::enrichParentGroups($groups);
 
 		order_result($groups, 'name');
 
@@ -407,34 +360,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			'with_triggers' => true,
 			'preservekeys' => true
 		]);
-
-		$parents = [];
-		$parent_name = '';
-		foreach ($groups as $group) {
-			$parent = explode('/', $group['name']);
-			if (count($parent) > 1) {
-				array_pop($parent);
-				foreach ($parent as $sub_parent) {
-					if ($parent_name === '') {
-						$parent_name = $sub_parent;
-					}
-					else {
-						$parent_name .= '/'.$sub_parent;
-					}
-					$parents[] = $parent_name;
-				}
-			}
-		}
-
-		if ($parents) {
-			$parent_groups = API::HostGroup()->get([
-				'output' => ['groupid', 'name'],
-				'filter' => ['name' => $parents],
-				'preservekeys' => true
-			]);
-
-			$groups = array_replace($groups, $parent_groups);
-		}
+		$groups = CPageFilter::enrichParentGroups($groups);
 
 		order_result($groups, 'name');
 
@@ -503,7 +429,17 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 
 	$triggers = API::Trigger()->get($triggerOptions);
 
-	CArrayHelper::sort($triggers, ['host', 'description']);
+	if (getRequest('filter_hostid') == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
+		foreach ($triggers as &$trigger) {
+			$trigger['host_name'] = $trigger['hosts'][0]['name'];
+		}
+		unset($trigger);
+
+		CArrayHelper::sort($triggers, ['host_name', 'description']);
+	}
+	else {
+		CArrayHelper::sort($triggers, ['description']);
+	}
 
 	$paging = getPagingLine($triggers, ZBX_SORT_UP, new CUrl('report2.php'));
 
@@ -514,7 +450,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 
 		$triggerTable->addRow([
 			($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE)
-				? $trigger['hosts'][0]['name'] : null,
+				? $trigger['host_name'] : null,
 			new CLink($trigger['description'],
 				(new CUrl('zabbix.php'))
 					->setArgument('action', 'problem.view')
